<?php

namespace Oro\Bundle\ProductBundle\DataGrid\EventListener\FrontendLineItemsGrid;

use Oro\Bundle\AttachmentBundle\Manager\AttachmentManager;
use Oro\Bundle\DataGridBundle\Datagrid\DatagridInterface;
use Oro\Bundle\DataGridBundle\Event\OrmResultAfter;
use Oro\Bundle\LocaleBundle\Formatter\NumberFormatter;
use Oro\Bundle\ProductBundle\Entity\Product;
use Oro\Bundle\ProductBundle\Model\ProductLineItemInterface;

/**
 * Adds data needed for displaying grouped line items.
 */
class LineItemsGroupedOnResultAfterListener
{
    /** @var AttachmentManager */
    private $attachmentManager;

    /** @var NumberFormatter */
    private $numberFormatter;

    /**
     * @param AttachmentManager $attachmentManager
     * @param NumberFormatter $numberFormatter
     */
    public function __construct(AttachmentManager $attachmentManager, NumberFormatter $numberFormatter)
    {
        $this->attachmentManager = $attachmentManager;
        $this->numberFormatter = $numberFormatter;
    }

    /**
     * @param OrmResultAfter $event
     */
    public function onResultAfter(OrmResultAfter $event): void
    {
        $isGrouped = $this->isGrouped($event->getDatagrid());
        if (!$isGrouped) {
            return;
        }

        foreach ($event->getRecords() as $record) {
            /** @var ProductLineItemInterface[] $lineItems */
            $lineItems = $record->getValue('lineItemsByIds') ?? [];
            if (count($lineItems) <= 1) {
                // Skips simple line items rows.
                continue;
            }

            $firstLineItem = reset($lineItems);
            if (!$firstLineItem instanceof ProductLineItemInterface) {
                throw new \LogicException(
                    sprintf(
                        'Element lineItemsByIds was expected to contain %s objects',
                        ProductLineItemInterface::class
                    )
                );
            }

            $parentProduct = $firstLineItem->getParentProduct();
            if (!$parentProduct instanceof Product) {
                throw new \LogicException('Property parentProduct was expected to be not null');
            }

            $parentProductId = $parentProduct->getId();
            $displayedLineItemsIds = explode(',', $record->getValue('displayedLineItemsIds') ?? '');
            $lineItemsData = $record->getValue('lineItemsDataByIds') ?? [];
            $firstLineItemData = reset($lineItemsData);
            $unitCode = $firstLineItem->getProductUnitCode();
            $configurableRecordId = $parentProductId . '_' . $unitCode;

            $recordData = [
                'id' => $configurableRecordId,
                'productId' => $parentProductId,
                'sku' => null,
                'image' => $this->getImageUrl($parentProduct),
                // It is assumed that first sub line item row already has a name of parent product.
                'name' => $firstLineItemData['name'] ?? '',
                'quantity' => 0,
                'unit' => $unitCode,
                // It is assumed that first sub line item already has currency set.
                'currency' => $firstLineItemData['currency'] ?? '',
                'subtotalValue' => 0,
                'discountValue' => 0,
                'subData' => [],
            ];

            foreach ($lineItemsData as $lineItemData) {
                $lineItemData['filteredOut'] = !in_array($lineItemData['id'] ?? 0, $displayedLineItemsIds, false);
                $recordData['subData'][] = $lineItemData;
            }

            $this->processSubtotal($recordData, $lineItemsData);

            foreach ($recordData as $name => $value) {
                $record->setValue($name, $value);
            }
        }
    }

    /**
     * @param array $recordData
     * @param array $lineItemsData
     */
    private function processSubtotal(array &$recordData, array $lineItemsData): void
    {
        foreach ($lineItemsData as $lineItemData) {
            $recordData['quantity'] += $lineItemData['quantity'] ?? 0;

            if (!isset($lineItemData['subtotalValue'], $lineItemData['currency'])) {
                $recordData['subtotalValue'] = $recordData['discountValue'] = null;
            }

            if ($recordData['subtotalValue'] !== null) {
                $recordData['subtotalValue'] += $lineItemData['subtotalValue'];
                if (isset($lineItemData['discountValue'])) {
                    $recordData['discountValue'] += $lineItemData['discountValue'];
                    // It is supposed that subtotalValue already includes applied discount, so we should revert it
                    // to get correct row initial subtotal of configurable product.
                    $recordData['subtotalValue'] += $lineItemData['discountValue'];
                }
            }
        }

        if ($recordData['subtotalValue']) {
            if ($recordData['discountValue']) {
                $recordData['discount'] = $this->formatCurrency('discountValue', $recordData);
                $recordData['initialSubtotal'] = $this->formatCurrency('subtotalValue', $recordData);
                $recordData['subtotalValue'] -= $recordData['discountValue'];
            }

            $recordData['subtotal'] = $this->formatCurrency('subtotalValue', $recordData);
        }
    }

    /**
     * @param string $key
     * @param array $lineItemData
     * @return string
     */
    private function formatCurrency(string $key, array $lineItemData): string
    {
        return $this->numberFormatter->formatCurrency($lineItemData[$key], $lineItemData['currency']);
    }

    /**
     * @param Product $product
     * @return string
     */
    private function getImageUrl(Product $product): string
    {
        $image = $product->getImagesByType('listing')->first();

        return $image ? $this->attachmentManager->getFilteredImageUrl($image->getImage(), 'product_small') : '';
    }

    /**
     * @param DatagridInterface $datagrid
     * @return bool
     */
    private function isGrouped(DatagridInterface $datagrid): bool
    {
        $parameters = $datagrid->getParameters()->get('_parameters', []);

        return isset($parameters['group']) ? filter_var($parameters['group'], FILTER_VALIDATE_BOOLEAN) : false;
    }
}
